<?php

declare(strict_types = 1);

namespace Drupal\file_extractor\Plugin\file_extractor\Extractor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\file\FileInterface;
use Drupal\file_extractor\Extractor\ExtractorPluginBase;

/**
 * Provides python pdf2text extractor.
 *
 * @FileExtractorExtractor(
 *   id = "python_pdf2txt_extractor",
 *   label = @Translation("Python Pdf2txt Extractor"),
 *   description = @Translation("Adds python Pdf2txt extractor support."),
 * )
 */
class PythonPdf2txtExtractor extends ExtractorPluginBase implements PluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'python_path' => 'python',
      'python_pdf2txt_script' => '/usr/bin/pdf2txt.py',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['python_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Python binary'),
      '#description' => $this->t('Enter the name of @binary executable or the full path to the @binary binary. Example: "@binary_example" or "@full_path_example".', [
        '@binary' => 'python',
        '@binary_example' => 'python',
        '@full_path_example' => '/usr/bin/python',
      ]),
      '#default_value' => $this->configuration['python_path'],
      '#required' => TRUE,
    ];
    $form['python_pdf2txt_script'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Python pdf2txt script'),
      '#description' => $this->t('Enter the full path to the Python pdf2txt script. Example: "/usr/bin/pdf2txt.py".'),
      '#default_value' => $this->configuration['python_pdf2txt_script'],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Check that the file exists.
    $python_pdf2txt_script = $form_state->getValue('python_pdf2txt_script');
    if (!file_exists($python_pdf2txt_script)) {
      $form_state->setError($form['python_pdf2txt_script'], $this->t('The file %path does not exist.', ['%path' => $python_pdf2txt_script]));
    }
    // Check that the file is an executable Python script.
    else {
      $python_path = $form_state->getValue('python_path');
      $cmd = escapeshellcmd($python_path) . ' ' . escapeshellarg($python_pdf2txt_script);
      exec($cmd, $output, $return_code);
      // $return_code = 1 if it fails. 100 instead.
      if ($return_code != 100) {
        $form_state->setError($form['python_path'], '');
        $form_state->setError($form['python_pdf2txt_script'], $this->t('Python Pdf2txt script file is not executable or there is a problem with the Python binary.'));
      }
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function extract(FileInterface $file) : string {
    // This extractor can only extract PDF.
    if (!in_array($file->getMimeType(), $this->getPdfMimeTypes())) {
      return '';
    }

    $log_variables = [];
    $python_path = $this->configuration['python_path'];
    $python_pdf2txt_script = realpath($this->configuration['python_pdf2txt_script']);
    $file_path = $this->getRealpath($file->getFileUri());

    // Check Python binary and Python pdf2txt script.
    if (!$this->checkBinaryAndFile($python_path, $python_pdf2txt_script, $log_variables)) {
      return '';
    }

    // Check file to extract. It will re-check Python binary; but this avoid
    // code duplication.
    if (!$this->checkBinaryAndFile($python_path, $file_path, $log_variables)) {
      return '';
    }

    $output = $this->execExtraction($python_path, [
      $python_pdf2txt_script,
      ' -C -t text ',
      $file_path,
    ]);
    if (is_null($output)) {
      $this->logger->error('An error occurred during the extraction of the file @file_path with the binary @binary_path.', $log_variables);
      return '';
    }
    return $output;
  }

}
